#include <R.h> 
#include <Rdefines.h>
#include <Rmath.h> 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* return the euclidean distance between (x1, y1)
   and (x2, y2) with wrap-around boundaries  */
double distance(double x1, double y1, double x2, double y2,
		double xlen, double ylen) {
  double x_diff, y_diff, d;
  x_diff = fabs(x1 - x2);
  y_diff = fabs(y1 - y2);
  /* make sure the x (respectively, y) distance is modulus the range
   limits x_len (respectively, y_len) */
  x_diff = (x_diff<xlen/2) ? x_diff : xlen-x_diff;
  y_diff = (y_diff<ylen/2) ? y_diff : ylen-y_diff;
  d = sqrt(x_diff * x_diff + y_diff * y_diff);
  return d;
}

/* compute all pairwise distances for individuals that are within
   crit_d of one another */
int compute_distances(double *x, double *y, double xlen, double ylen, 
		      int *n, int *out_i, int *out_j, double *out_val,
		      int len, double crit_d) {
  int N = *n, i, j, k;
  double d;

  for ( i=0, k=0; i<N; i++ )
    for ( j=i+1; j<N; j++ ) {
      d = distance(x[i], y[i], x[j], y[j], xlen, ylen);
      if ( d < crit_d ) {
	out_i[k] = i+1; /* +1 so indexing is as in R */
	out_j[k] = j+1; /* +1 so indexing is as in R */
	out_val[k] = d;
	k++;
	if ( k >= len ) {
	  /* we need a larger sparse matrix size so we will now
	     compute what size this should be and then start over with
	     filling it in.  This only happens a few times at the
	     beginning of the simulation while we figure out
	     approximately what size sparse matrices are needed. */
	  for ( i=0, k=0; i<N; i++ )
	    for ( j=i+1; j<N; j++ ) {
	      if ( distance(x[i], y[i], x[j], y[j], xlen, ylen) < crit_d ) {
		k++;
	      }
	    }
	  return k;
	}
      }
    }
  return k;
}

/* construct sparse matrix of pairwise distances */
SEXP distances(SEXP X, SEXP Y, SEXP XLEN, SEXP YLEN,
	       SEXP LEN, SEXP CRIT_D) {
  int n = LENGTH(X), len = *INTEGER(LEN), len_new;
  double crit_d = *REAL(CRIT_D);
  double x_len = *REAL(XLEN);
  double y_len = *REAL(YLEN);
  /* objects to return */
  SEXP out_i, out_j, out_val, r_len;
  /* list which will contain the above objects */
  SEXP ret;

  PROTECT(out_i = allocVector(INTSXP, len));
  PROTECT(out_j = allocVector(INTSXP, len));
  PROTECT(out_val = allocVector(REALSXP, len));

  len_new = compute_distances(REAL(X),
			      REAL(Y),
			      x_len,
			      y_len,
			      &n,
			      INTEGER(out_i),
			      INTEGER(out_j),
			      REAL(out_val),
			      len,
			      crit_d);

  if ( len_new > len ) {
    UNPROTECT(3);
    PROTECT(out_i = allocVector(INTSXP, len_new));
    PROTECT(out_j = allocVector(INTSXP, len_new));
    PROTECT(out_val = allocVector(REALSXP, len_new));
    len_new = compute_distances(REAL(X),
				REAL(Y),
				x_len,
				y_len,
				&n,
				INTEGER(out_i),
				INTEGER(out_j),
				REAL(out_val),
				len_new,
				crit_d);
  }

/*   some helpful print statements for debugging: */
/*   Rprintf("n = %d\n", n); */
/*   Rprintf("len = %d\n", len); */
/*   Rprintf("len_new = %d\n", len_new); */
/*   Rprintf("length(out_i) = %d\n", LENGTH(out_i)); */
/*   Rprintf("crit_d = %2.3f\n", crit_d); */

  /* build the list and return, including len_new */
  PROTECT(r_len = allocVector(INTSXP, 1));
  INTEGER(r_len)[0] = len_new;
  PROTECT(ret = allocVector(VECSXP, 4));
  SET_VECTOR_ELT(ret, 0, out_i);
  SET_VECTOR_ELT(ret, 1, out_j);
  SET_VECTOR_ELT(ret, 2, out_val);
  SET_VECTOR_ELT(ret, 3, r_len);
  UNPROTECT(5);
  return ret;
}
