#include <R.h> 
#include <Rdefines.h>
#include <Rmath.h> 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "getListElement.h"

/* return the mating probability of female i with male j */

/* two-locus allelic model with 2 types */
/* negative alpha implies complete assortment */
double mp_2loc_allelic(double trait_dist, double spatial_dist,
		       double alpha, double sigma_f) {
  double res;
  if(alpha<0) {
    res = (1-trait_dist) *
      exp(-(spatial_dist*spatial_dist)/(2*sigma_f*sigma_f) );
  }
  else {
    res = pow(1/alpha, trait_dist) *
      exp(-(spatial_dist*spatial_dist)/(2*sigma_f*sigma_f) );
  }
  return res;
}

/* two-locus allelic model with n types */
/* negative alpha implies complete assortment */
double mp_2loc_allelic_ntypes(double trait_dist, double spatial_dist,
			       double alpha, double sigma_f) {
  if(trait_dist>0) trait_dist = 1;
  double res;
  if(alpha<0) {
    res = (1-trait_dist) *
      exp(-(spatial_dist*spatial_dist)/(2*sigma_f*sigma_f) );
  }
  else {
    res = pow(1/alpha, trait_dist) *
      exp(-(spatial_dist*spatial_dist)/(2*sigma_f*sigma_f) );
  }
  return res;
}

/* two-locus quantitative model */
double mp_2loc_quant(double trait_dist, double spatial_dist,
		     double sigma_p, double sigma_f) {
  double res;

  res = exp(-(trait_dist*trait_dist)/(2*sigma_p*sigma_p)
	    -(spatial_dist*spatial_dist)/(2*sigma_f*sigma_f));
  return res;
}

int compute_mate_probs(int dimension, 
		       int genetic_model, 
		       int num_loci,
		       int num_types,
		       int pop_size,
		       double alpha,
		       double sigma_f,
		       double sigma_p,
		       double *disp,
		       double *pref,
		       int *sex,
		       int *x,
		       int *y,
		       double *dist,
		       int dist_size,
		       double *male_fitness,
		       int *out_female,
		       int *out_male,
		       double *out_val) {

  int i, k;
  double trait_dist;
  double mf, mp;
  double denom;

  if(dimension==1) denom = sqrt(2 * PI) * sigma_f;
  else denom = 2 * PI * sigma_f * sigma_f;

  /* the -1s below are necessary so that indexing is correct for C
     which begins at 0 (instead of 1 as occurs in R) */
  for ( i=0, k=0; i<dist_size; i++ ) {
    if(sex[x[i]-1]!=sex[y[i]-1]) {
      if(sex[x[i]-1]==0) {
	out_male[k]=x[i];
	out_female[k]=y[i];
      }
      else {
	out_male[k]=y[i];
	out_female[k]=x[i];
      }

      trait_dist = fabs(pref[(out_female[k]-1)] -
			disp[(out_male[k]-1)]);

      /* allelic model */
      if(genetic_model==1) {
	/* two genotypes */
	if(num_loci==1 && num_types==2)
	  mp = mp_2loc_allelic(trait_dist, dist[i],
			      alpha, sigma_f);
	/* num_types genotypes */
	if(num_loci==1 && num_types>2)
	  mp = mp_2loc_allelic_ntypes(trait_dist, dist[i],
				      alpha, sigma_f);
      }

      /* quantitative model */
      if(genetic_model==2)
	mp = mp_2loc_quant(trait_dist, dist[i],
			   sigma_p, sigma_f);

      /* weight mating probabilities by male fitnesses (these
	 fitnesses are all equal to 1 in the survival model where
	 resource shares do not affect mating success) */
      out_val[k] = male_fitness[out_male[k]-1] * mp / denom;
      k++;
    }
  }
  return k;
}

/* return the mating probabilities sparse matrix */
SEXP mate_probs(SEXP prms,
		SEXP pop,
		SEXP distances,
		SEXP fitnesses) {

  int dimension = *INTEGER(getListElement(prms, "dimension"));
  int genetic_model = *INTEGER(getListElement(prms, "genetic.model"));
  int num_loci = *INTEGER(getListElement(prms, "num.loci"));
  int num_types = *INTEGER(getListElement(prms, "num.types"));
  int pop_size = *INTEGER(getListElement(pop, "pop.size"));
  double alpha = *REAL(getListElement(prms, "alpha"));
  double sigma_f = *REAL(getListElement(prms, "sigma.f"));
  double sigma_p = *REAL(getListElement(prms, "sigma.p"));
  double *disp, *pref; int *sex;
  disp = REAL(getListElement(pop, "disp"));
  pref = REAL(getListElement(pop, "pref"));
  sex = INTEGER(getListElement(pop, "sex"));

  int *x, *y;
  double *dist;
  int dist_size;
  int mat_size;
  dist_size = *INTEGER(getListElement(distances, "size"));
  x = INTEGER(getListElement(distances, "i"));
  y = INTEGER(getListElement(distances, "j"));
  dist = REAL(getListElement(distances, "distance"));

  double *male_fitness;
  male_fitness = REAL(fitnesses);

  SEXP out_female, out_male, out_val;
  PROTECT(out_female = allocVector(INTSXP, dist_size));
  PROTECT(out_male = allocVector(INTSXP, dist_size));
  PROTECT(out_val = allocVector(REALSXP, dist_size));
  mat_size = compute_mate_probs(dimension,
				genetic_model,
				num_loci,
				num_types,
				pop_size,
				alpha,
				sigma_f,
				sigma_p,
				disp,
				pref,
				sex,
				x, y,
				dist,
				dist_size,
				male_fitness,
				INTEGER(out_female),
				INTEGER(out_male),
				REAL(out_val));

  /* build the list and return, including len_new */
  SEXP r_mat_size, ret;
  PROTECT(r_mat_size = allocVector(INTSXP, 1));
  INTEGER(r_mat_size)[0] = mat_size;
  PROTECT(ret = allocVector(VECSXP, 4));
  SET_VECTOR_ELT(ret, 0, out_female);
  SET_VECTOR_ELT(ret, 1, out_male);
  SET_VECTOR_ELT(ret, 2, out_val);
  SET_VECTOR_ELT(ret, 3, r_mat_size);
  UNPROTECT(5);
  return ret;
}
