## load source functions
source("src_r/patchy.R")
source("src_r/c_functions.R")
source("src_r/fitnesses.R")
source("src_r/move.R")
source("src_r/death.R")
source("src_r/birth.R")
source("src_r/variation_functions.R")
source("src_r/plotting.R")
source("src_r/meta_functions.R")
dyn.load("src_c/distances.so")
dyn.load("src_c/mate_probs.so")
dyn.load("src_c/comp_coeffs.so")

## ******************************************************************
## *********************** store parameters *************************
## ******************************************************************
prms.f <- function(dimension,
                   spatial.model,
                   genetic.model,
                   comp.model,
                   num.loci,
                   move.model,
                   asymmetric.costs,
                   num.types,
                   sigma.s,
                   sigma.f,
                   sigma.m,
                   sigma.p,
                   sigma.r,
                   recomb.rate,
                   N,
                   K,
                   alpha,
                   fem.cost,
                   cost.type.1,
                   s0,
                   x.len,
                   y.len,
                   fxy,
                   male.move,
                   num.gens,
                   init.spatial.config,
                   print.every,
                   file.name) {

  ## critical distance for optimization is at least 3 times the
  ## standard deviation of interaction width, etc
  s.D <- 3*max(sigma.m, sigma.s, sigma.f)

  ## create normalized K(x,y)
  if(spatial.model==1) {
    if(dimension==1) area <- x.len
    if(dimension==2) area <- x.len*y.len
    fxy.norm <- function(x,y) 1/area
  }
  if(spatial.model==2) {
    grid.size <- 500
    seq.x <- 0:(x.len*grid.size)/grid.size
    seq.y <- 0:(y.len*grid.size)/grid.size
    x.y <- expand.grid(seq.x, seq.y)
    if(dimension==1) {
      area <- x.len
      mean.height <- sum(fxy(x=seq.x, y=0))/length(seq.x)
    }
    if(dimension==2) {
      area <- x.len*y.len
      mean.height <- sum(fxy(x=x.y[,1], y=x.y[,2]))/(nrow(x.y))
    }
    integral.K <- mean.height*area
    fxy.norm <- function(x,y) fxy(x,y)/integral.K
  }
  
  ## compute Holling parameter 2 (not a free variable)
  w0 = s0 - K/N

  ## set sparse matrix optimization parameter
  init.sparse.dim = as.integer(1)
  
  list(dimension=dimension,
       spatial.model=spatial.model,
       genetic.model=genetic.model,
       comp.model=comp.model,
       num.loci=num.loci,
       move.model=move.model,
       asymmetric.costs=asymmetric.costs,
       num.types=num.types,
       sigma.s=sigma.s,
       sigma.f=sigma.f,
       sigma.m=sigma.m,
       sigma.p=sigma.p,
       sigma.r=sigma.r,
       recomb.rate=recomb.rate,
       N=N,
       K=K,
       alpha=alpha,
       fem.cost=fem.cost,
       cost.type.1=cost.type.1,
       s0=s0,
       w0=w0,
       x.len=x.len,
       y.len=y.len,
       fxy=fxy.norm,
       male.move=male.move,
       num.gens=num.gens,
       init.spatial.config=init.spatial.config,
       init.sparse.dim=init.sparse.dim,
       crit.distance=s.D,
       print.every=print.every,
       file.name=file.name)
}

## ******************************************************************
## **************** initialize the population ***********************
## ******************************************************************
initialize.pop <- function(prms) { 
  ## assign individuals random initial locations
  x <- runif(prms$N)*prms$x.len
  y <- runif(prms$N)*prms$y.len
  if(prms$dimension==1) y <- rep(0.25, prms$N)

  ## random initial distribution of genotypes
  if(prms$init.spatial.config==1) {
    disp <- matrix(floor(runif(prms$N*prms$num.loci)*prms$num.types),
                   ncol=prms$num.loci)
    pref <- matrix(floor(runif(prms$N*prms$num.loci)*prms$num.types),
                   ncol=prms$num.loci)
  }

  ## half and half initial distribution of genotypes (for two-loci) or
  ## strips of genotypes for more than two
  if(prms$init.spatial.config==2) {
    disp <- matrix(0, nrow=prms$N*prms$num.loci, ncol=prms$num.loci)
    pref <- matrix(0, nrow=prms$N*prms$num.loci, ncol=prms$num.loci)
    for(i in 1:(prms$num.types-1)) {
      disp[x<(prms$x.len*(prms$num.types-i)/prms$num.types),] <- i
      pref[x<(prms$x.len*(prms$num.types-i)/prms$num.types),] <- i
    }
  }
  
  list(x=x, y=y,
       sex=as.integer(1*(runif(prms$N)<0.5)), ## 0==male, 1==female
       disp=disp, pref=pref,
       pop.size=prms$N)
}
